;external Keyboard LEDs
;for use with wireless keyboards with LED-less receivers
;more information: http://www.elektronik.de.vu/

;ATTiny11L @ 1MHz (internal R/C)
;Fuses: Default Settings

;PORTS:
;  PB0 = LED Scroll Lock
;  PB1 = LED Caps Lock
;  PB2 = LED Num Lock
;  PB3 = Data
;  PB4 = Clock
;
;VERSION HISTORY:
;  0.6  2006-04-15  Arne Rossius
;     * first working version

.include "../include/tn11def.inc"

.equ	SCROLL = 0
.equ	CAPS = 1
.equ	NUM = 2
.equ	CLK = 4
.equ	DATA = 3

.def	temp = R16
.def	temp2 = R17
.def	count = R18
.def	bits1 = R19
.def	bits2 = R20
.def	oldcmd = R21

;===============================================================================

.cseg
.org 0x000
	rjmp	start
	
start:
	;init ports
	ldi	temp, 1<<SCROLL | 1<<CAPS | 1<<NUM
	out	DDRB, temp
	ldi	temp, 1<<SCROLL | 1<<CAPS | 1<<NUM
	out	PORTB, temp

	;init registers
	ldi	temp, 1<<CLK | 1<<DATA
	ldi	oldcmd, 0x00

;--------------------

loop:
	;wait for negative edge on DATA while clock is low
	mov	temp2, temp
	in	temp, PINB
	andi	temp, 1<<CLK | 1<<DATA
	cpi	temp, 0x00
	brne	loop
	cpi	temp2, 1<<DATA
	brne	loop

	;receive 12 bits
	clr	bits1
	clr	bits2
	ldi	count, 12
rec1:
	sbis	PINB, CLK
	rjmp	rec1
rec2:
	sbic	PINB, CLK
	rjmp	rec2
	clc
	sbic	PINB, DATA
	sec
	ror	bits2
	ror	bits1
	dec	count
	brne	rec1

	;align so the data byte is in bits1
	ror	bits2
	ror	bits1
	ror	bits2
	ror	bits1
	ror	bits2
	ror	bits1
	ror	bits2
	ror	bits1
	ror	bits2
	ror	bits1

	;check if "set LEDs" command has been sent
	cpi	oldcmd, 0xED ;0xED = set LEDs
	breq	set_leds
	mov	oldcmd, bits1
	rjmp	loop

set_leds:
	;set the 3 LEDs
	ldi	bits2, 0x00
	sbrs	bits1, 0 ;Scroll Lock
	ori	bits2, 1<<SCROLL
	sbrs	bits1, 1 ;Num Lock
	ori	bits2, 1<<NUM
	sbrs	bits1, 2 ;Caps Lock
	ori	bits2, 1<<CAPS
	out	PORTB, bits2
	ldi	oldcmd, 0x00
	rjmp	loop
